$col0 = 0;
$col1 = 3;
$col2 = 6;
$col3 = 9;
$col4 = 12;
$col5 = 15;

function makePowernets()
{
  if ($Sim::Time - $LastPowernetz < 0.5)
    return;

  $LastPowernetz = $Sim::Time;
  PowernetGroup.deleteAll();
  %count = WireGroup.getCount();

  for (%i = 0; %i < %count; %i++)
  {
    %wire = WireGroup.getObject(%i);

    if (isObject(%wire.powernet))
      continue;

    %net = createPowernet(%wire);
    %net.propagateMembership(%wire);
  }

  //bottomPrintAll("\c3" @ PowernetGroup.getCount() @ " powernets");
}

function createPowernet(%wire)
{
  if (isObject(%wire.powernet))
    return 0;

  %obj = new ScriptObject()
  {
    class = PowernetSO;
  };

  PowernetGroup.add(%obj);

  %wire.powernet = %obj;
  %obj.wires.add(%wire);

  return %obj;
}

function PowernetSO::onAdd(%this)
{
  %this.load = 0;
  %this.avail = 0;

  %this.nextLoad = 0;
  %this.nextAvail = 0;

  %this.viewLoad = 0;
  %this.excess = 0;

  %this.wires = new SimSet();
  %this.machines = new SimSet();
}

function PowernetSO::onRemove(%this)
{
  %this.wires.delete();
  %this.machines.delete();
}

function PowernetSO::propagateMembership(%this, %wire)
{
  %links = %wire.getPowerLinks();

  if (%links $= "")
    return;

  %count = getWordCount(%links);

  for (%i = 0; %i < %count; %i++)
  {
    %link = getWord(%links, %i);

    if (isObject(%link.powernet))
      continue;

    %link.powernet = %this;
    %this.wires.add(%link);

    %this.propagateMembership(%link);
  }
}

function PowernetSO::tick(%this)
{
  %this.load = %this.nextLoad;
  %this.avail = %this.nextAvail;

  %this.nextLoad = 0;
  %this.nextAvail = 0;

  %this.viewLoad = 0.8 * %this.viewLoad + 0.2 * %this.load;
  %this.viewLoad = mFloatLength(%this.viewLoad, 0);

  %this.excess = %this.avail - %this.load;
  %this.store += %this.excess / 32;

  if (%this.store < 0)
    %this.store = 0;

  if (%this.store > 100000)
    %this.store = 100000;
}

function PowernetSO::drain(%this, %watts)
{
  %this.nextLoad += %watts;
}

function powernetTick()
{
  cancel($powernetTick);
  %count = PowernetGroup.getCount();

  for (%i = 0; %i < %count; %i++)
  {
    %obj = PowernetGroup.getObject(%i);

    %obj.nextLoad = %obj.load + getRandom(-30, 30);
    %obj.nextLoad = mClampF(%obj.nextLoad, 500, 2000);

    %obj.nextAvail = %obj.avail + getRandom(-30, 30);
    %obj.nextAvail = mClampF(%obj.nextAvail, 500, 2000);

    %obj.tick();
  }

  if (%count)
  {
    %obj = PowernetGroup.getObject(0);

    %text = "\c3" @ %count SPC "powernets - first stats:\n";
    %text = %text @ "\c3Viewload (load over time): " @ %obj.viewLoad @ "W\n";
    %text = %text @ "\c3Excess (unused available power): " @ %obj.excess @ "W\n";
    %text = %text @ "\c3Store (direct-SMES cell): " @ %obj.store @ "J\n";

    bottomPrintAll(%text, 0, 1);
  }

  $powernetTick = schedule(32, 0, powernetTick);
}

if (!isObject(PowernetGroup))
  new SimSet(PowernetGroup);

if (!isEventPending($powernetTick))
  $powernetTick = schedule(100, 0, powernetTick);

function scalar()
{
  return getRandom() * 2 - 1;
}
